@@ -59,15 +59,9 @@ NS_IMPL_CYCLE_COLLECTION_TRACE_END
 NS_IMPL_CYCLE_COLLECTING_ADDREF(DOMSVGTransformList)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(DOMSVGTransformList)
 
-} // namespace mozilla
-DOMCI_DATA(SVGTransformList, mozilla::DOMSVGTransformList)
-namespace mozilla {
-
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(DOMSVGTransformList)
   NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
-  NS_INTERFACE_MAP_ENTRY(nsIDOMSVGTransformList)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(SVGTransformList)
 NS_INTERFACE_MAP_END
 
 //----------------------------------------------------------------------
@@ -129,24 +123,6 @@ DOMSVGTransformList::InternalList() const
 }
 
 //----------------------------------------------------------------------
-// nsIDOMSVGTransformList methods:
-
-/* readonly attribute unsigned long numberOfItems; */
-NS_IMETHODIMP
-DOMSVGTransformList::GetNumberOfItems(uint32_t *aNumberOfItems)
-{
-  *aNumberOfItems = NumberOfItems();
-  return NS_OK;
-}
-
-/* readonly attribute unsigned long length; */
-NS_IMETHODIMP
-DOMSVGTransformList::GetLength(uint32_t *aLength)
-{
-  *aLength = Length();
-  return NS_OK;
-}
-
 void
 DOMSVGTransformList::Clear(ErrorResult& error)
 {
@@ -171,17 +147,8 @@ DOMSVGTransformList::Clear(ErrorResult& error)
   }
 }
 
-/* void clear (); */
-NS_IMETHODIMP
-DOMSVGTransformList::Clear()
-{
-  ErrorResult rv;
-  Clear(rv);
-  return rv.ErrorCode();
-}
-
-already_AddRefed<nsIDOMSVGTransform>
-DOMSVGTransformList::Initialize(nsIDOMSVGTransform *newItem, ErrorResult& error)
+already_AddRefed<DOMSVGTransform>
+DOMSVGTransformList::Initialize(DOMSVGTransform& newItem, ErrorResult& error)
 {
   if (IsAnimValList()) {
     error.Throw(NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR);
@@ -196,31 +163,17 @@ DOMSVGTransformList::Initialize(nsIDOMSVGTransform *newItem, ErrorResult& error)
   // clone of newItem, it would actually insert newItem. To prevent that from
   // happening we have to do the clone here, if necessary.
 
-  nsCOMPtr<DOMSVGTransform> domItem = do_QueryInterface(newItem);
-  if (!domItem) {
-    error.Throw(NS_ERROR_DOM_SVG_WRONG_TYPE_ERR);
-    return nullptr;
-  }
+  nsCOMPtr<DOMSVGTransform> domItem = &newItem;
   if (domItem->HasOwner()) {
-    newItem = domItem->Clone();
+    domItem = newItem.Clone();
   }
 
   Clear(error);
   MOZ_ASSERT(!error.Failed(), "How could this fail?");
-  return InsertItemBefore(newItem, 0, error);
+  return InsertItemBefore(*domItem, 0, error);
 }
 
-/* nsIDOMSVGTransform initialize (in nsIDOMSVGTransform newItem); */
-NS_IMETHODIMP
-DOMSVGTransformList::Initialize(nsIDOMSVGTransform *newItem,
-                                nsIDOMSVGTransform **_retval)
-{
-  ErrorResult rv;
-  *_retval = Initialize(newItem, rv).get();
-  return rv.ErrorCode();
-}
-
-nsIDOMSVGTransform*
+DOMSVGTransform*
 DOMSVGTransformList::IndexedGetter(uint32_t index, bool& found,
                                    ErrorResult& error)
 {
@@ -235,17 +188,8 @@ DOMSVGTransformList::IndexedGetter(uint32_t index, bool& found,
   return nullptr;
 }
 
-/* nsIDOMSVGTransform getItem (in unsigned long index); */
-NS_IMETHODIMP
-DOMSVGTransformList::GetItem(uint32_t index, nsIDOMSVGTransform **_retval)
-{
-  ErrorResult rv;
-  NS_IF_ADDREF(*_retval = GetItem(index, rv));
-  return rv.ErrorCode();
-}
-
-already_AddRefed<nsIDOMSVGTransform>
-DOMSVGTransformList::InsertItemBefore(nsIDOMSVGTransform *newItem,
+already_AddRefed<DOMSVGTransform>
+DOMSVGTransformList::InsertItemBefore(DOMSVGTransform& newItem,
                                       uint32_t index, ErrorResult& error)
 {
   if (IsAnimValList()) {
@@ -259,13 +203,9 @@ DOMSVGTransformList::InsertItemBefore(nsIDOMSVGTransform *newItem,
     return nullptr;
   }
 
-  nsCOMPtr<DOMSVGTransform> domItem = do_QueryInterface(newItem);
-  if (!domItem) {
-    error.Throw(NS_ERROR_DOM_SVG_WRONG_TYPE_ERR);
-    return nullptr;
-  }
-  if (domItem->HasOwner()) {
-    domItem = domItem->Clone(); // must do this before changing anything!
+  nsCOMPtr<DOMSVGTransform> domItem = &newItem;
+  if (newItem.HasOwner()) {
+    domItem = newItem.Clone(); // must do this before changing anything!
   }
 
   // Ensure we have enough memory so we can avoid complex error handling below:
@@ -296,20 +236,8 @@ DOMSVGTransformList::InsertItemBefore(nsIDOMSVGTransform *newItem,
   return domItem.forget();
 }
 
-/* nsIDOMSVGTransform insertItemBefore (in nsIDOMSVGTransform newItem,
- *                                      in unsigned long index); */
-NS_IMETHODIMP
-DOMSVGTransformList::InsertItemBefore(nsIDOMSVGTransform *newItem,
-                                      uint32_t index,
-                                      nsIDOMSVGTransform **_retval)
-{
-  ErrorResult rv;
-  *_retval = InsertItemBefore(newItem, index, rv).get();
-  return rv.ErrorCode();
-}
-
-already_AddRefed<nsIDOMSVGTransform>
-DOMSVGTransformList::ReplaceItem(nsIDOMSVGTransform *newItem,
+already_AddRefed<DOMSVGTransform>
+DOMSVGTransformList::ReplaceItem(DOMSVGTransform& newItem,
                                  uint32_t index, ErrorResult& error)
 {
   if (IsAnimValList()) {
@@ -317,17 +245,14 @@ DOMSVGTransformList::ReplaceItem(nsIDOMSVGTransform *newItem,
     return nullptr;
   }
 
-  nsCOMPtr<DOMSVGTransform> domItem = do_QueryInterface(newItem);
-  if (!domItem) {
-    error.Throw(NS_ERROR_DOM_SVG_WRONG_TYPE_ERR);
-    return nullptr;
-  }
   if (index >= LengthNoFlush()) {
     error.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
     return nullptr;
   }
-  if (domItem->HasOwner()) {
-    domItem = domItem->Clone(); // must do this before changing anything!
+
+  nsCOMPtr<DOMSVGTransform> domItem = &newItem;
+  if (newItem.HasOwner()) {
+    domItem = newItem.Clone(); // must do this before changing anything!
   }
 
   nsAttrValue emptyOrOldValue = Element()->WillChangeTransformList();
@@ -351,19 +276,7 @@ DOMSVGTransformList::ReplaceItem(nsIDOMSVGTransform *newItem,
   return domItem.forget();
 }
 
-/* nsIDOMSVGTransform replaceItem (in nsIDOMSVGTransform newItem,
- *                                 in unsigned long index); */
-NS_IMETHODIMP
-DOMSVGTransformList::ReplaceItem(nsIDOMSVGTransform *newItem,
-                                 uint32_t index,
-                                 nsIDOMSVGTransform **_retval)
-{
-  ErrorResult rv;
-  *_retval = ReplaceItem(newItem, index, rv).get();
-  return rv.ErrorCode();
-}
-
-already_AddRefed<nsIDOMSVGTransform>
+already_AddRefed<DOMSVGTransform>
 DOMSVGTransformList::RemoveItem(uint32_t index, ErrorResult& error)
 {
   if (IsAnimValList()) {
@@ -388,7 +301,7 @@ DOMSVGTransformList::RemoveItem(uint32_t index, ErrorResult& error)
   // Notify the DOM item of removal *before* modifying the lists so that the
   // DOM item can copy its *old* value:
   mItems[index]->RemovingFromList();
-  nsCOMPtr<nsIDOMSVGTransform> result = mItems[index];
+  nsCOMPtr<DOMSVGTransform> result = mItems[index];
 
   InternalList().RemoveItem(index);
   mItems.RemoveElementAt(index);
@@ -402,51 +315,14 @@ DOMSVGTransformList::RemoveItem(uint32_t index, ErrorResult& error)
   return result.forget();
 }
 
-/* nsIDOMSVGTransform removeItem (in unsigned long index); */
-NS_IMETHODIMP
-DOMSVGTransformList::RemoveItem(uint32_t index, nsIDOMSVGTransform **_retval)
+already_AddRefed<DOMSVGTransform>
+DOMSVGTransformList::CreateSVGTransformFromMatrix(DOMSVGMatrix& matrix)
 {
-  ErrorResult rv;
-  *_retval = RemoveItem(index, rv).get();
-  return rv.ErrorCode();
-}
-
-/* nsIDOMSVGTransform appendItem (in nsIDOMSVGTransform newItem); */
-NS_IMETHODIMP
-DOMSVGTransformList::AppendItem(nsIDOMSVGTransform *newItem,
-                                nsIDOMSVGTransform **_retval)
-{
-  ErrorResult rv;
-  *_retval = AppendItem(newItem, rv).get();
-  return rv.ErrorCode();
-}
-
-already_AddRefed<nsIDOMSVGTransform>
-DOMSVGTransformList::CreateSVGTransformFromMatrix(nsIDOMSVGMatrix *matrix,
-                                                  ErrorResult& error)
-{
-  nsCOMPtr<DOMSVGMatrix> domItem = do_QueryInterface(matrix);
-  if (!domItem) {
-    error.Throw(NS_ERROR_DOM_SVG_WRONG_TYPE_ERR);
-    return nullptr;
-  }
-
-  nsCOMPtr<nsIDOMSVGTransform> result = new DOMSVGTransform(domItem->Matrix());
+  nsCOMPtr<DOMSVGTransform> result = new DOMSVGTransform(matrix.Matrix());
   return result.forget();
 }
 
-/* nsIDOMSVGTransform createSVGTransformFromMatrix (in nsIDOMSVGMatrix matrix);
- */
-NS_IMETHODIMP
-DOMSVGTransformList::CreateSVGTransformFromMatrix(nsIDOMSVGMatrix *matrix,
-                                                  nsIDOMSVGTransform **_retval)
-{
-  ErrorResult rv;
-  *_retval = CreateSVGTransformFromMatrix(matrix, rv).get();
-  return rv.ErrorCode();
-}
-
-already_AddRefed<nsIDOMSVGTransform>
+already_AddRefed<DOMSVGTransform>
 DOMSVGTransformList::Consolidate(ErrorResult& error)
 {
   if (IsAnimValList()) {
@@ -454,8 +330,9 @@ DOMSVGTransformList::Consolidate(ErrorResult& error)
     return nullptr;
   }
 
-  if (LengthNoFlush() == 0)
+  if (LengthNoFlush() == 0) {
     return nullptr;
+  }
 
   // Note that SVG 1.1 says, "The consolidation operation creates new
   // SVGTransform object as the first and only item in the list" hence, even if
@@ -471,16 +348,7 @@ DOMSVGTransformList::Consolidate(ErrorResult& error)
 
   // And append the new transform
   nsRefPtr<DOMSVGTransform> transform = new DOMSVGTransform(mx);
-  return InsertItemBefore(transform, LengthNoFlush(), error);
-}
-
-/* nsIDOMSVGTransform consolidate (); */
-NS_IMETHODIMP
-DOMSVGTransformList::Consolidate(nsIDOMSVGTransform **_retval)
-{
-  ErrorResult rv;
-  *_retval = Consolidate(rv).get();
-  return rv.ErrorCode();
+  return InsertItemBefore(*transform, LengthNoFlush(), error);
 }
 
 //----------------------------------------------------------------------
