@@ -45,6 +45,8 @@ nsEmptyStyleRule::MapRuleInfoInto(nsRuleData* aRuleData)
 /* virtual */ void
 nsEmptyStyleRule::List(FILE* out, int32_t aIndent) const
 {
+  for (int32_t index = aIndent; --index >= 0; ) fputs("  ", out);
+  fputs("[empty style rule] {}\n", out);
 }
 #endif
 
@@ -89,6 +91,8 @@ nsInitialStyleRule::MapRuleInfoInto(nsRuleData* aRuleData)
 /* virtual */ void
 nsInitialStyleRule::List(FILE* out, int32_t aIndent) const
 {
+  for (int32_t index = aIndent; --index >= 0; ) fputs("  ", out);
+  fputs("[initial style rule] {}\n", out);
 }
 #endif
 
@@ -512,7 +516,8 @@ ReplaceAnimationRule(nsRuleNode *aOldRuleNode,
   }
 
   NS_ABORT_IF_FALSE(!IsMoreSpecificThanAnimation(n) &&
-                    n->GetLevel() != nsStyleSet::eAnimationSheet,
+                    (n->IsRoot() ||
+                     n->GetLevel() != nsStyleSet::eAnimationSheet),
                     "wrong level");
 
   if (aNewAnimRule) {
